@@ -21,9 +21,24 @@ else
 fi
 
 
-NVIDIA_BUG_REPORT_CHANGE="$Change: 1833133 $"
+NVIDIA_BUG_REPORT_CHANGE="$Change: 7677613 $"
 NVIDIA_BUG_REPORT_VERSION=`echo $NVIDIA_BUG_REPORT_CHANGE | tr -c -d [:digit:]`
 
+#
+# echo_metadata() - echo metadata of specified file
+#
+
+echo_metadata() {
+    printf "*** ls: "
+    /bin/ls -l -D "%Y-%m-%d %H.%M.%S" "$1" 2> /dev/null
+
+    if [ $? -ne 0 ]; then
+        # Run dumb ls -l. We might not get one-second mtime granularity, but
+        # that is probably okay.
+        ls -l "$1"
+    fi
+}
+
 #
 # append() - append the contents of the specified file to the log
 #
@@ -34,11 +49,12 @@ append() {
         echo ""
 
         if [ ! -f "$1" ]; then
-            echo "$1 does not exist"
+            echo "*** $1 does not exist"
         elif [ ! -r "$1" ]; then
-            echo "$1 is not readable"
+            echo "*** $1 is not readable"
         else
-            echo "$1"
+            echo "*** $1"
+            echo_metadata "$1"
             cat  "$1"
         fi
         echo ""
@@ -55,7 +71,8 @@ append_silent() {
         if [ -f "$1" -a -r "$1" ]; then
             echo "____________________________________________"
             echo ""
-            echo "$1"
+            echo "*** $1"
+            echo_metadata "$1"
             cat  "$1"
             echo ""
         fi
@@ -103,8 +120,7 @@ append_sysctl() {
 #
 
 
-# check that we are root (needed for `lspci -vxxx` and potentially for
-# accessing kernel log files)
+# check that we are root (potentially needed for accessing kernel log files)
 
 if [ `id -u` -ne 0 ]; then
     echo "ERROR: Please run $(basename $0) as root."
@@ -361,7 +377,9 @@ done
     echo "nvidia-settings -q all:"
     echo ""
 
-    nvidia-settings -q all 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1
+    DPY="$DISPLAY"
+    [ "$DPY" ] || DPY=":0"
+    DISPLAY= nvidia-settings -c "$DPY" -q all 2>&1 & sleep 1 ; kill -9 $! > /dev/null 2>&1    
 
     if [ $? -eq 0 ]; then
         # The nvidia-settings process is still there.
