@@ -97,6 +97,7 @@
 #include <sys/sx.h>
 #include <sys/condvar.h>
 #include <sys/sglist.h>
+#include <sys/taskqueue.h>
 
 #define CURTHREAD curthread
 
@@ -183,6 +184,12 @@ struct nvidia_filep {
     struct selinfo event_rsel;
 } nvidia_filep_t;
 
+typedef
+struct nvidia_work {
+    struct task task;
+    void *data;
+} nvidia_work_t;
+
 typedef
 struct nvidia_softc {
     device_t dev;
@@ -347,9 +354,9 @@ void   nvidia_rc_timer       (void *);
 void   nv_lock_api           (nv_state_t *);
 void   nv_unlock_api         (nv_state_t *);
 
-NvS32  nv_alloc_contig_pages (nv_state_t *, NvU32, NvU32, NvU64 *, void **);
+NvS32  nv_alloc_contig_pages (nv_state_t *, NvU32, NvU32, NvBool, NvU64 *, void **);
 NvS32  nv_free_contig_pages  (nv_state_t *, void *);
-NvS32  nv_alloc_system_pages (nv_state_t *, NvU32, NvU32, NvU64 *, void **);
+NvS32  nv_alloc_system_pages (nv_state_t *, NvU32, NvU32, NvBool, NvU64 *, void **);
 NvS32  nv_free_system_pages  (nv_state_t *, void *);
 
 NvS32  nv_os_agp_init        (nv_stack_t *sp, nv_state_t *, NvU64 *, NvU64 *);
