@@ -409,7 +409,8 @@ typedef struct
 ** where we hide our nv_state_t * ...
 */
 #define NV_SET_NV_STATE(pgpu,p) ((pgpu)->pOsHwInfo = (p))
-#define NV_GET_NV_STATE(pgpu)   ((nv_state_t *)(pgpu)->pOsHwInfo)
+#define NV_GET_NV_STATE(pGpu) \
+    (nv_state_t *)((pGpu) ? (pGpu)->pOsHwInfo : NULL)
 
 /* mmap(2) offsets */
 
@@ -525,7 +526,7 @@ void*  NV_API_CALL  nv_get_adapter_state         (NvU32, NvU16, NvU16);
 void   NV_API_CALL  nv_set_dma_address_size      (nv_state_t *, NvU32 );
 
 RM_STATUS  NV_API_CALL  nv_alias_pages           (nv_state_t *, NvU32, NvU32, NvU32, NvU64, NvU64 *, void **);
-RM_STATUS  NV_API_CALL  nv_alloc_pages           (nv_state_t *, NvU32, NvU32, NvU32, NvU32, NvU64 *, void **);
+RM_STATUS  NV_API_CALL  nv_alloc_pages           (nv_state_t *, NvU32, NvBool, NvBool, NvU32, NvBool, NvU64 *, void **);
 RM_STATUS  NV_API_CALL  nv_free_pages            (nv_state_t *, NvU32, NvU32, NvU32, NvU32, void *);
 RM_STATUS  NV_API_CALL  nv_guest_pfn_list        (nv_state_t *, unsigned int, unsigned int, unsigned int, unsigned int*);
 
@@ -551,7 +552,11 @@ void   NV_API_CALL  nv_acpi_methods_uninit       (void);
 RM_STATUS  NV_API_CALL  nv_acpi_method           (NvU32, NvU32, NvU32, void *, NvU16, NvU32 *, void *, NvU16 *);
 RM_STATUS  NV_API_CALL  nv_acpi_dsm_method       (nv_state_t *, NvU8 *, NvU32, NvU32, void *, NvU16, NvU32 *, void *, NvU16 *);
 RM_STATUS  NV_API_CALL  nv_acpi_ddc_method       (nv_state_t *, void *, NvU32 *);
-void*  NV_API_CALL  nv_get_smu_state             (void); 
+void*  NV_API_CALL  nv_get_smu_state             (void);
+RM_STATUS  NV_API_CALL  nv_lock_user_pages       (nv_state_t *, void *, NvU64, NvU64 *, void **);
+RM_STATUS  NV_API_CALL  nv_unlock_user_pages     (nv_state_t *, NvU64, NvU64 *, void *);
+RM_STATUS  NV_API_CALL  nv_acpi_rom_method       (nv_state_t *, NvU32 *, NvU32 *);
+RM_STATUS  NV_API_CALL  nv_log_error             (nv_state_t *, NvU32, const char *, void *);
 
 #ifdef NV_USER_MAP
 #include "nv-map.h"
@@ -604,8 +609,10 @@ RM_STATUS  NV_API_CALL  rm_read_registry_dword   (nv_stack_t *, nv_state_t *, Nv
 RM_STATUS  NV_API_CALL  rm_write_registry_dword  (nv_stack_t *, nv_state_t *, NvU8 *, NvU8 *, NvU32);
 RM_STATUS  NV_API_CALL  rm_read_registry_binary  (nv_stack_t *, nv_state_t *, NvU8 *, NvU8 *, NvU8 *, NvU32 *);
 RM_STATUS  NV_API_CALL  rm_write_registry_binary (nv_stack_t *, nv_state_t *, NvU8 *, NvU8 *, NvU8 *, NvU32);
+RM_STATUS  NV_API_CALL  rm_write_registry_string (nv_stack_t *, nv_state_t *, NvU8 *, NvU8 *, const char *, NvU32);
 
 RM_STATUS  NV_API_CALL  rm_run_rc_callback       (nv_stack_t *, nv_state_t *);
+void       NV_API_CALL  rm_execute_work_item     (nv_stack_t *, void *);
 RM_STATUS  NV_API_CALL  rm_get_device_name       (nv_stack_t *, nv_state_t *, NvU16, NvU16, NvU16, NvU32, NvU8 *);
 
 NvU64      NV_API_CALL  nv_rdtsc                 (void);
