@@ -13,27 +13,12 @@
 #include "nv.h"
 #include "nv-freebsd.h"
 
-
-/*
- * The NVIDIA kernel module's malloc identifier, needed for both tracking
- * and actual allocation/freeing purposes. M_NVIDIA is declared elsewhere
- * to make it known to other parts of the kernel module (nv-freebsd.h).
- */
-
 MALLOC_DEFINE(M_NVIDIA, "nvidia", "NVIDIA memory allocations");
-
-
-#define MAX_ERROR_STRING    256
-
-/*
- * The core resource manager's favorite source of memory, this routine is
- * called from different contexts, including ISRs. This means that it can
- * not be allowed to sleep when memory is low.
- */
+TASKQUEUE_DEFINE_THREAD(nvidia);
 
 RM_STATUS NV_API_CALL os_alloc_mem(
     void **address,
-    U032 size
+    NvU32 size
 )
 {
     /* XXX Fix me? (malloc flags) */
@@ -51,7 +36,7 @@ void NV_API_CALL os_free_mem(void *address)
 #define NV_USECS_PER_TICK       (1000000 / hz)
 #define NV_USECS_TO_TICKS(us)   ((us) * hz / 1000000)
 
-RM_STATUS NV_API_CALL os_delay(U032 MilliSeconds)
+RM_STATUS NV_API_CALL os_delay(NvU32 MilliSeconds)
 {
     unsigned long MicroSeconds;
     unsigned long ticks;
@@ -94,7 +79,7 @@ RM_STATUS NV_API_CALL os_delay(U032 MilliSeconds)
     return RM_OK;
 }
 
-RM_STATUS NV_API_CALL os_delay_us(U032 MicroSeconds)
+RM_STATUS NV_API_CALL os_delay_us(NvU32 MicroSeconds)
 {
     if (__NV_ITHREAD() && (MicroSeconds > NV_MAX_ISR_DELAY_US))
         return RM_ERROR;
@@ -102,12 +87,13 @@ RM_STATUS NV_API_CALL os_delay_us(U032 MicroSeconds)
     return RM_OK;
 }
 
-U032 NV_API_CALL os_get_cpu_frequency(void)
+NvU64 NV_API_CALL os_get_cpu_frequency(void)
 {
-    return ((tsc_freq + 4999) / 1000000);
+    /* round up by 4999 before division by 1000000 in osGetCpuFrequency() */
+    return (tsc_freq + 4999);
 }
 
-U032 NV_API_CALL os_get_current_process(void)
+NvU32 NV_API_CALL os_get_current_process(void)
 {
     return curproc->p_pid;
 }
@@ -123,8 +109,8 @@ RM_STATUS NV_API_CALL os_get_current_thread(NvU64 *threadId)
 }
 
 RM_STATUS NV_API_CALL os_get_current_time(
-    U032 *sec,
-    U032 *usec
+    NvU32 *sec,
+    NvU32 *usec
 )
 {
     struct timeval tv;
@@ -142,8 +128,8 @@ BOOL NV_API_CALL os_is_administrator(PHWINFO pDev)
     return suser(CURTHREAD) ? FALSE : TRUE;
 }
 
-U008 NV_API_CALL os_io_read_byte(
-    U032 address
+NvU8 NV_API_CALL os_io_read_byte(
+    NvU32 address
 )
 {
     /* XXX Fix me? (bus_space access) */
@@ -151,16 +137,16 @@ U008 NV_API_CALL os_io_read_byte(
 }
 
 void NV_API_CALL os_io_write_byte(
-    U032 address,
-    U008 value
+    NvU32 address,
+    NvU8  value
 )
 {
     /* XXX Fix me? (bus_space access) */
     outb(address, value);
 }
 
-U016 NV_API_CALL os_io_read_word(
-    U032 address
+NvU16 NV_API_CALL os_io_read_word(
+    NvU32 address
 )
 {
     /* XXX Fix me? (bus_space access) */
@@ -168,16 +154,16 @@ U016 NV_API_CALL os_io_read_word(
 }
 
 void NV_API_CALL os_io_write_word(
-    U032 address,
-    U016 value
+    NvU32 address,
+    NvU16 value
 )
 {
     /* XXX Fix me? (bus_space access) */
     return outw(address, value);
 }
 
-U032 NV_API_CALL os_io_read_dword(
-    U032 address
+NvU32 NV_API_CALL os_io_read_dword(
+    NvU32 address
 )
 {
     /* XXX Fix me? (bus_space access) */
@@ -185,8 +171,8 @@ U032 NV_API_CALL os_io_read_dword(
 }
 
 void NV_API_CALL os_io_write_dword(
-    U032 address,
-    U032 value
+    NvU32 address,
+    NvU32 value
 )
 {
     /* XXX Fix me? (bus_space access) */
@@ -196,7 +182,7 @@ void NV_API_CALL os_io_write_dword(
 void* NV_API_CALL os_map_kernel_space(
     NvU64 start,
     NvU64 size,
-    U032 mode
+    NvU32 mode
 )
 {
     int map_mode;
@@ -242,7 +228,7 @@ void NV_API_CALL os_unmap_kernel_space(
 RM_STATUS NV_API_CALL os_set_mem_range(
     NvU64 start,
     NvU64 size,
-    U032 mode
+    NvU32 mode
 )
 {
     int arg;
@@ -286,11 +272,11 @@ RM_STATUS NV_API_CALL os_unset_mem_range(
  * highest debug level, i.e. the lowest debug output.
  */
 
-U032 cur_debuglevel = 0xffffffff;
+NvU32 cur_debuglevel = 0xffffffff;
 
 void NV_API_CALL os_dbg_init(void)
 {
-    U032 new_debuglevel;
+    NvU32 new_debuglevel;
     nv_stack_t *sp;
 
     NV_UMA_ZONE_ALLOC_STACK(sp);
@@ -311,7 +297,47 @@ RM_STATUS NV_API_CALL os_schedule(void)
     return RM_ERR_NOT_SUPPORTED;
 }
 
-void NV_API_CALL os_dbg_set_level(U032 new_debuglevel)
+static void os_execute_work_item(void *context, int pending)
+{
+    nvidia_work_t *work = (nvidia_work_t *)context;
+    nv_stack_t *sp = NULL;
+
+    NV_UMA_ZONE_ALLOC_STACK(sp);
+    if (sp == NULL) {
+        nv_printf(NV_DBG_ERRORS, "NVRM: failed to allocate stack!\n");
+        return;
+    }
+
+    rm_execute_work_item(sp, work->data);
+    os_free_mem((void *)work);
+
+    NV_UMA_ZONE_FREE_STACK(sp);
+}
+
+RM_STATUS NV_API_CALL os_queue_work_item(void *data)
+{
+    RM_STATUS status;
+    nvidia_work_t *work;
+
+    status = os_alloc_mem((void **)&work, sizeof(nvidia_work_t));
+    if (status != status)
+        return status;
+
+    work->data = data;
+
+    TASK_INIT(&work->task, 0, os_execute_work_item, (void *)work);
+    taskqueue_enqueue(taskqueue_nvidia, &work->task);
+
+    return RM_OK;
+}
+
+RM_STATUS NV_API_CALL os_flush_work_queue(void)
+{
+    taskqueue_run(taskqueue_nvidia);
+    return RM_OK;
+}
+
+void NV_API_CALL os_dbg_set_level(NvU32 new_debuglevel)
 {
     cur_debuglevel = new_debuglevel;
 }
@@ -328,10 +354,11 @@ void NV_API_CALL out_string(const char *message)
     printf("%s", message);
 }
 
+#define MAX_ERROR_STRING 256
 static char nv_error_string[MAX_ERROR_STRING];
 
 int NV_API_CALL nv_printf(
-    U032 debuglevel,
+    NvU32 debuglevel,
     const char *format,
     ...
 )
@@ -350,9 +377,9 @@ int NV_API_CALL nv_printf(
     return chars_written;
 }
 
-int NV_API_CALL nv_snprintf(
+NvS32 NV_API_CALL os_snprintf(
     char *buf,
-    unsigned int size,
+    NvU32 size,
     const char *fmt,
     ...
 )
@@ -367,8 +394,9 @@ int NV_API_CALL nv_snprintf(
     return chars_written;
 }
 
-void NV_API_CALL nv_os_log(
-    int loglevel,
+void NV_API_CALL os_log_error(
+    NvU32 log_level,
+    NvU32 error_number,
     const char *fmt,
     void *ap
 )
@@ -380,19 +408,19 @@ void NV_API_CALL nv_os_log(
     printf("%s", nv_error_string);
 }
 
-S032 NV_API_CALL os_mem_cmp(
-    const U008 *buf0,
-    const U008 *buf1,
-    U032 length
+NvS32 NV_API_CALL os_mem_cmp(
+    const NvU8 *buf0,
+    const NvU8 *buf1,
+    NvU32 length
 )
 {
     return memcmp(buf0, buf1, length);
 }
 
-U008* NV_API_CALL os_mem_copy(
-    U008 *dst,
-    const U008 *src,
-    U032 length
+NvU8* NV_API_CALL os_mem_copy(
+    NvU8 *dst,
+    const NvU8 *src,
+    NvU32 length
 )
 {
 #if defined(NVCPU_X86_64)
@@ -407,7 +435,7 @@ U008* NV_API_CALL os_mem_copy(
 RM_STATUS NV_API_CALL os_memcpy_from_user(
     void *dst,
     const void *src,
-    U032 length
+    NvU32 length
 )
 {
     if (src < (void *) VM_MAXUSER_ADDRESS)
@@ -419,7 +447,7 @@ RM_STATUS NV_API_CALL os_memcpy_from_user(
 RM_STATUS NV_API_CALL os_memcpy_to_user(
     void *dst,
     const void *src,
-    U032 length
+    NvU32 length
 )
 {
     if (dst < (void *) VM_MAXUSER_ADDRESS)
@@ -430,8 +458,8 @@ RM_STATUS NV_API_CALL os_memcpy_to_user(
 
 void* NV_API_CALL os_mem_set(
     void *b,
-    U008 c,
-    U032 length
+    NvU8  c,
+    NvU32 length
 )
 {
     return memset(b, c, length);
@@ -445,7 +473,7 @@ char* NV_API_CALL os_string_copy(
     return strcpy(dst, src);
 }
 
-U032 NV_API_CALL os_string_length(const char* s)
+NvU32 NV_API_CALL os_string_length(const char* s)
 {
     return strlen(s);
 }
@@ -453,13 +481,13 @@ U032 NV_API_CALL os_string_length(const char* s)
 RM_STATUS NV_API_CALL os_strncpy_from_user(
     char *dst,
     const char *src,
-    U032 n
+    NvU32 n
 )
 {
     return copyinstr(src, dst, n, NULL) ? RM_ERR_INVALID_POINTER : RM_OK;
 }
 
-U032 NV_API_CALL os_get_page_size(void)
+NvU32 NV_API_CALL os_get_page_size(void)
 {
     return PAGE_SIZE;
 }
@@ -474,7 +502,7 @@ NvU64 NV_API_CALL os_get_system_memory_size(void)
     return ((NvU64)physmem * PAGE_SIZE) / RM_PAGE_SIZE;
 }
 
-U032 NV_API_CALL os_get_cpu_count(void)
+NvU32 NV_API_CALL os_get_cpu_count(void)
 {
     return mp_ncpus;
 }
@@ -520,6 +548,14 @@ RM_STATUS NV_API_CALL os_clear_smp_barrier(void)
     return RM_ERR_NOT_SUPPORTED;
 }
 
+void NV_API_CALL os_disable_preemption(void)
+{
+}
+
+void NV_API_CALL os_enable_preemption(void)
+{
+}
+
 struct os_mutex {
     nv_stack_t *sp;
     struct mtx mutex_mtx;
@@ -629,15 +665,15 @@ BOOL NV_API_CALL os_pat_supported(void)
 }
 
 void NV_API_CALL os_register_compatible_ioctl(
-    U032 cmd,
-    U032 size
+    NvU32 cmd,
+    NvU32 size
 )
 {
 }
 
 void NV_API_CALL os_unregister_compatible_ioctl(
-    U032 cmd,
-    U032 size
+    NvU32 cmd,
+    NvU32 size
 )
 {
 }
