@@ -66,12 +66,14 @@ void        NV_API_CALL  os_free_mem                 (void *);
 RM_STATUS   NV_API_CALL  os_get_current_time         (NvU32 *, NvU32 *);
 RM_STATUS   NV_API_CALL  os_delay                    (NvU32);
 RM_STATUS   NV_API_CALL  os_delay_us                 (NvU32);
-NvU32       NV_API_CALL  os_get_cpu_frequency        (void);
+NvU64       NV_API_CALL  os_get_cpu_frequency        (void);
 NvU32       NV_API_CALL  os_get_current_process      (void);
 RM_STATUS   NV_API_CALL  os_get_current_thread       (NvU64 *);
 char*       NV_API_CALL  os_string_copy              (char *, const char *);
 RM_STATUS   NV_API_CALL  os_strncpy_from_user        (char *, const char *, NvU32);
 NvU32       NV_API_CALL  os_string_length            (const char *);
+NvS32       NV_API_CALL  os_snprintf                 (char *, NvU32, const char *, ...);
+void        NV_API_CALL  os_log_error                (NvU32, NvU32, const char *, void *);
 NvU8*       NV_API_CALL  os_mem_copy                 (NvU8 *, const NvU8 *, NvU32);
 RM_STATUS   NV_API_CALL  os_memcpy_from_user         (void *, const void *, NvU32);
 RM_STATUS   NV_API_CALL  os_memcpy_to_user           (void *, const void *, NvU32);
@@ -104,6 +106,8 @@ void        NV_API_CALL  os_dbg_set_level            (NvU32);
 NvU32       NV_API_CALL  os_get_cpu_count            (void);
 RM_STATUS   NV_API_CALL  os_raise_smp_barrier        (void);
 RM_STATUS   NV_API_CALL  os_clear_smp_barrier        (void);
+void        NV_API_CALL  os_disable_preemption       (void);
+void        NV_API_CALL  os_enable_preemption        (void);
 RM_STATUS   NV_API_CALL  os_disable_console_access   (void);
 RM_STATUS   NV_API_CALL  os_enable_console_access    (void);
 RM_STATUS   NV_API_CALL  os_registry_init            (void);
@@ -117,7 +121,8 @@ RM_STATUS   NV_API_CALL  os_schedule                 (void);
 NvU64       NV_API_CALL  os_acquire_spinlock         (void *);
 void        NV_API_CALL  os_release_spinlock         (void *, NvU64);
 RM_STATUS   NV_API_CALL  os_get_address_space_info   (NvU64 *, NvU64 *, NvU64 *, NvU64 *);
-
+RM_STATUS   NV_API_CALL  os_queue_work_item          (void *);
+RM_STATUS   NV_API_CALL  os_flush_work_queue         (void);
 void        NV_API_CALL  os_register_compatible_ioctl    (NvU32, NvU32);
 void        NV_API_CALL  os_unregister_compatible_ioctl  (NvU32, NvU32);
 
@@ -166,9 +171,6 @@ void        NV_API_CALL  os_dump_stack (void);
 void NV_API_CALL  out_string(const char *str);
 int  NV_API_CALL  nv_printf(NvU32 debuglevel, const char *printf_format, ...);
 void NV_API_CALL  nv_prints(NvU32 debuglevel, const char *string);
-int  NV_API_CALL  nv_snprintf(char *buf, unsigned int size, const char *fmt, ...);
-void NV_API_CALL  nv_os_log(int log_level, const char *fmt, void *ap);
-
 
 #define NV_MEMORY_TYPE_SYSTEM       0
 #define NV_MEMORY_TYPE_AGP          1
